/** @file KInstruction.h
* Basic script instructions
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 05.09.15 - First version
* 06.03.16 - Linux version
*
*/

#pragma once


namespace K3d
{
	/// Enumerations describing the bytecode instructions the VM is able to process
    enum EScriptOpCode
    {
        K_OP_TALK,
		K_OP_PRINT,

		/// Variable manipulators
        K_OP_SET,	///< char, char : destination index, value to set
        K_OP_INC,	///< char : index to increment
        K_OP_DEC,	///< char : index to decrement
        K_OP_ADD,	///< char, char, char : dest index, srce index1, srce index2
        K_OP_END,	///< End script variable
		K_NUM_CODES	///< Number of opcodes if first opcode has value of 0
    };

	class CInstruction 
	{
		EScriptOpCode  m_eCode;	///< Enumerations describing the bytecode instructions
		CBuffer  m_kData;
	public:
		///\brief Constructor 
		///\param _eCode Enumerations describing the bytecode instructions
		///\see EScriptOpCode
		CInstruction(EScriptOpCode _eCode) : m_eCode(_eCode)  {}
        CInstruction(EScriptOpCode _eCode, const char* _strData, size_t _tDataSize)
            : m_eCode(_eCode), m_kData(_strData, _tDataSize)  {}
		~CInstruction(){}
	
		///\brief Get code enumerator
		///\retval m_eCode Enumerations describing the bytecode instructions
		EScriptOpCode GetCode() const         
		{ 
			return m_eCode; 
		}
		
		///\brief Get string data from buffer
		///\retval m_strData String data
		const char* GetData() const
		{ 
			return m_kData.GetData(); 
		}
	};
}	/// End namespace K3d
