/** @file KReadFile.h
* File handle used only for binary-mode reading.
* Provides an interface for reading basic data-types and strings.
* Dynamic-binding not supported.
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 06.03.23 - First version
* 
*
*/

#pragma once

namespace K3d
{
	class CReaderFile : public CFile
	{
	public:
		CReaderFile()	{}
		CReaderFile(const std::string& _rFileName)	
		{ 
			Open(_rFileName); 
		}
		///\brief Open file
		///\param _rFileName Reference to string filename
		///\retval bool True if file correctly opened
		bool Open(const std::string& _rFileName)	
		{ 
			return OpenForRead(_rFileName); 
		}
		///\brief Read file
		///\param _strBuf String buffer
		///\param _tLen File length
		///\retval int Total number of readed items
		///\retval _strBuf File string
		int Read(char* _strBuf, size_t _tLen)	
		{ 
			assert(Good()); 
			return fread(reinterpret_cast<void*>(_strBuf), sizeof(char), _tLen, GetFile()); 
		}
		///\brief Read string
		///\param _rStr Reference to string
		///\retval int Total number of readed items
		int Read(std::string& _rStr)
		{ 
			long lLen; 
			int iVal = Read(lLen); 
			_rStr.resize(lLen); 
			iVal += Read(&_rStr[0], lLen); 
			return lLen; 
		}
		///\brief Read char
		///\param _rcVal Reference to char
		///\retval int Total number of readed items
		int Read(char& _rcVal)
		{ 
			return Read(reinterpret_cast<char*>(&_rcVal), sizeof(char)); 
		}
		///\brief Read unsigned char
		///\param _rucVal Reference to unsigned char
		///\retval int Total number of readed items
		int Read(unsigned char& _rucVal)
		{ 
			return Read(reinterpret_cast<char*>(&_rucVal), sizeof(unsigned char)); 
		}
		///\brief Read short
		///\param _rsVal Reference to short
		///\retval int Total number of readed items
		int Read(short& _rsVal)	
		{ 
			return Read(reinterpret_cast<char*>(&_rsVal), sizeof(short)); 
		}
		///\brief Read unsigned short
		///\param _rsVal Reference to unsigned short
		///\retval int Total number of readed items
		int Read(unsigned short& _rusVal)		
		{ 
			return Read(reinterpret_cast<char*>(&_rusVal), sizeof(unsigned short)); 
		}
		///\brief Read long
		///\param _rlVal Reference to long
		///\retval int Total number of readed items
		int Read(long& _rlVal)					
		{ 
			return Read(reinterpret_cast<char*>(&_rlVal), sizeof(long)); 
		}
		///\brief Read unsigned long
		///\param _rulVal Reference to unsigned long
		///\retval int Total number of readed items
		int Read(unsigned long& _rulVal)		
		{ 
			return Read(reinterpret_cast<char*>(&_rulVal), sizeof(unsigned long));
		}
		///\brief Read short
		///\param _tVal Reference to size_t
		///\retval int Total number of readed items
		int Read(size_t& _tVal)				
		{ 
			return Read(reinterpret_cast<char*>(&_tVal), sizeof(size_t)); 
		}
		///\brief Read float
		///\param _rfVal Reference to float
		///\retval int Total number of readed items
		int Read(float& _rfVal)
		{ 
			return Read(reinterpret_cast<char*>(&_rfVal), sizeof(float)); 
		}
		///\brief Read double
		///\param _rdVal Reference to float
		///\retval int Total number of readed items
		int Read(double& _rdVal)
		{ 
			return Read(reinterpret_cast<char*>(&_rdVal), sizeof(double)); 
		}
	};
}	/// End namespace K3d
