/** @file KScanner.h
* Converts a string into an array of meaningful values
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 06.03.22 - First version
*
*
*/

#pragma once



namespace K3d
{
    typedef std::vector<char>           TCharArray;
    typedef std::vector<std::string>    TStringArray;

    class CScanner
    {
		TStringArray m_tCodeNameArray;
		TCharArray m_tTokArray;
		size_t m_tOffset;
    public:
        CScanner(const TStringArray& _rtCodeNames) : m_tCodeNameArray(_rtCodeNames)   {}
        CScanner(const std::string* _pCodeNameArray, size_t _tNumNames)
        {
            m_tCodeNameArray.resize(_tNumNames);
            const std::string* pStrArray = _pCodeNameArray+_tNumNames;
            std::copy(_pCodeNameArray, pStrArray, m_tCodeNameArray.begin());
        }
		///\brief Tokenize a line and return success
        bool ScanLine(const std::string& _rstrLine);
        ///\brief Get tokens from most recent scan
		const TCharArray& GetTokens() const
		{ 
			return m_tTokArray; 
		}  
    private:
		///\brief Skipping space in string
		///\param _rstrLine Reference to string line
		///\retval bool True if not empty string
		bool SkipSpacing(const std::string& _rLine)
        {
			while (isspace(_rLine.c_str()[m_tOffset]))
			{
                ++m_tOffset;
			}
			if (_rLine.c_str()[m_tOffset] == 0)
			{
                return false;
			}
            return true;
        }
		///\brief Scan alphabetic string
		///\param _rstrLine Reference to string line
		///\retval bool True if string line is found in code
		bool ScanCode(const std::string& _rLine)
        {
            size_t tBegin = m_tOffset;
			// Test alphabetic string
			while (isalpha(_rLine.c_str()[m_tOffset]))
			{
                ++m_tOffset;
			}
			return FindCode(std::string(_rLine, tBegin, m_tOffset-tBegin));
        }
		///\brief Scan numeric string
		///\param _rstrLine Reference to string line
		///\retval bool True if string line is found in code
        bool ScanNum(const std::string& _rLine)
        {
            size_t tBegin = m_tOffset;
			// Test numeric string
			while (isdigit(_rLine.c_str()[m_tOffset]))
			{
                ++m_tOffset;
			}
			// Were any digits scanned?
            if (m_tOffset == tBegin)
			{
                return false;
			}
			std::string strNumber(_rLine, tBegin, m_tOffset-tBegin);
            m_tTokArray.push_back(static_cast<char>(atoi(strNumber.c_str())));
            return true;
        }
		///\brief Find string in code
		///\param _rstr Reference to search string
		///\retval bool True if string is found in code
        bool FindCode(const std::string& _rStr)
        {
            char cCodeVal;
            TStringArray::iterator it = m_tCodeNameArray.begin();
            for (cCodeVal = 0; it != m_tCodeNameArray.end(); ++it, ++cCodeVal)
            {
				if (_rStr == *it)
                {
                    m_tTokArray.push_back(cCodeVal);
                    return true;
                }
            }
            return false;
        }
    };
}



