/** @file KTextReadFile.h
* File handle for reading in text-based mode.
* Dynamic-binding not supported.
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 06.03.23 - First version
* 
*
*/

#pragma once

namespace K3d
{
class CTextReaderFile : public CFile
{
	public:
		CTextReaderFile()	{}
		CTextReaderFile(const std::string& _rFileName)	
		{ 
			Open(_rFileName); 
		}
		///\brief Open for read text
		///\param _rFileName Referenct to file
		///\retval bool True if file correctly opened
		bool Open(const std::string& _rFileName)	
		{ 
			return OpenForReadText(_rFileName); 
		}
		///\brief Read text file
		///\param _strBuffer String buffer
		///\param _tLen File length
		///\retval int Total number of readed items
		///\retval _strBuffer File string
		int Read(char* _strBuffer, size_t _tLen)		
		{ 
			return fread(reinterpret_cast<void*>(_strBuffer), sizeof(char), _tLen, GetFile()); 
		}
		///\brief Read text file
		///\param _rStr String buffer
		///\param _tLen File length
		///\retval int Total number of readed items
		///\retval _rStr File string
		int Read(std::string& _rStr, size_t _tLen)	
		{ 
			_rStr.resize(_tLen); 
			return fread(reinterpret_cast<void*>(&_rStr[0]), sizeof(char), _tLen, GetFile()); 
		}
};
}	/// End namespace K3d
