/** @file KTextWriteFile.h
* TextWriterFile
* File handle for writing in text-based mode.
* Dynamic-binding not supported.
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 06.03.23 - First version
* 
*
*/

#pragma once

namespace K3d
{
	class CTextWriterFile : public CFile
	{
	public:
		CTextWriterFile()	{}
		CTextWriterFile(const std::string& _rFileName)	
		{ 
			Open(_rFileName); 
		}
		CTextWriterFile(const std::string& _rFileName, bool _bAppend)	
		{ 
			Open(_rFileName, _bAppend); 
		}
		///\brief Open for read text
		///\param _rFileName Referenct to file
		///\retval bool True if file correctly opened
		bool Open(const std::string &_rFileName)	
		{
			return OpenForWriteTextTrunc(_rFileName); 
		}
		///\brief Open for read text
		///\param _rFileName Referenct to file
		///\param _bAppend Open for append write?
		///\retval bool True if file correctly opened
		bool Open(const std::string& _rFileName, bool _bAppend)	
		{ 
			if (_bAppend)
			{
				return OpenForWriteTextAppend(_rFileName); 
			}
			else 
			{
				return OpenForWriteTextTrunc(_rFileName); 
			}
		}
		///\brief Write text file
		///\param _str String buffer
		///\retval int Total number of readed items
		///\retval _str File string
		int Write(const char* _str)
		{ 
			size_t tLen = strlen(_str); 
			return fwrite(reinterpret_cast<const void*>(_str), sizeof(char), tLen, GetFile()); 
		}
		///\brief Write text file
		///\param _str String buffer
		///\retval int Total number of readed items
		///\retval _str File string
		int Write(const std::string& _str)	
		{ 
			return fwrite(reinterpret_cast<const void*>(_str.data()), sizeof(char), _str.length(), GetFile()); 
		}
	};
}	/// End namespace K3d
