/** @file KVirtualMachine.h
* Script virtual machine VM
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 05.09.15 - First version
* 06.03.16 - Linux version
*
*/

#include "stdafx.h"			
using namespace K3d;

///\brief Execute VM
///\param _uiScriptId Script index
void CVirtualMachine::ExecuteScript(size_t _uiScriptId)
{
	// Select root script by script index
	SelectScript(_uiScriptId);	
	// Initialize variable data
	m_kState.SetDataSize(m_tScript->GetNumVars());
	// Set our iterator to the beginning       
	m_tInstrCurr = m_tInstrRoot;	
	while (m_tInstrCurr)
	{
		switch(m_tInstrCurr->GetCode())
		{
		// Message functionality
		case K_OP_TALK:
			std::cout << "I am talking." << std::endl;
			// Iterate
			++m_tInstrCurr;	
			break;
		case K_OP_PRINT:
			std::cout << m_tInstrCurr->GetData() << std::endl;    // print data
			++m_tInstrCurr;   // iterate
			break;
		// new semi-math functionality
		case K_OP_SET:
			assert(m_tInstrCurr->GetData()[0] < m_tScript->GetNumVars());
			m_kState.SetVar(m_tInstrCurr->GetData()[0], m_tInstrCurr->GetData()[1]);
			++m_tInstrCurr;
			break;
		case K_OP_INC:
			assert(m_tInstrCurr->GetData()[0] < m_tScript->GetNumVars());
			m_kState.SetVar(m_tInstrCurr->GetData()[0], m_kState.GetVar(m_tInstrCurr->GetData()[0])+1);
			++m_tInstrCurr;
			break;
		case K_OP_DEC:
			assert(m_tInstrCurr->GetData()[0] < m_tScript->GetNumVars());
			m_kState.SetVar(m_tInstrCurr->GetData()[0], m_kState.GetVar(m_tInstrCurr->GetData()[0])-1);
			++m_tInstrCurr;
			break;
		case K_OP_ADD:
			assert(m_tInstrCurr->GetData()[0] < m_tScript->GetNumVars());
			assert(m_tInstrCurr->GetData()[1] < m_tScript->GetNumVars());
			assert(m_tInstrCurr->GetData()[2] < m_tScript->GetNumVars());
			m_kState.SetVar(m_tInstrCurr->GetData()[0],
							m_kState.GetVar(m_tInstrCurr->GetData()[1])
							+ m_kState.GetVar(m_tInstrCurr->GetData()[2]));
			++m_tInstrCurr;
		case K_OP_END:
			// Discontinue the loop
			m_tInstrCurr = 0;	
			break;
		}
	}
}

///\brief Expose variable state
///\param _rState Reference to state
void CVirtualMachine::ExposeVariableState(const CScriptState& _rState) const
{
	std::vector<char>::const_iterator itr;
	int n = 0;  // Used to denote indexed position of value
	for (itr = _rState.GetDataArray().begin(); itr != _rState.GetDataArray().end(); ++itr, ++n)
	{
		std::cout << n << ": ";
		std::cout << static_cast<int>(*itr);   // Cast for numeric value
		std::cout << std::endl;
	}
}
