/** @file KVirtualMachine.h
* Script virtual machine VM
*
*	K3dEngine 1.0 - OpenGL Game Engine
*	http://sourceforge.net/projects/kengine/
*
* 05.09.15 - First version
* 06.03.16 - Linux version
*
*/

#pragma once

namespace K3d
{
	class CVirtualMachine
	{
		/// Useful abstractions
		/// pointers used as non-modifying dynamic references
		typedef const CScript* TScriptRef;
		typedef const CInstruction*  TInstrRef;

		CScriptState m_kState;	///< Script state. Since we're single-threaded, only need one
		std::vector<CScript> m_vScript;	///< Script array 
		TScriptRef m_tScript;		///< Current script
		TInstrRef m_tInstrRoot;		///< Root instruction
		TInstrRef m_tInstrCurr;	    ///< Current instruction
		size_t m_uiNumScripts;		///< Number of loaded scripts

		///\brief Add script to list and retrieve id
		///\param _rScript Reference to script object
		size_t AddScript(const CScript& _rScript) 
		{
			m_vScript.push_back(_rScript); 
			return m_uiNumScripts++;
		}
		///\brief Set current script by id
		///\param _uiScriptId Script index
		void SelectScript(size_t _uiScriptId) 
		{
			assert(_uiScriptId < m_uiNumScripts);  // make sure the id is valid
			m_tScript = &m_vScript[_uiScriptId];
			m_tInstrRoot = m_tScript->GetInstruction();
		}
		void ExposeVariableState(const CScriptState& _rState) const;
	
	public:
		CVirtualMachine()
			: m_tScript(0), m_tInstrRoot(0), m_tInstrCurr(0), m_uiNumScripts(0){}
		~CVirtualMachine(){}

		void ExecuteScript(size_t _uiScriptId);
		///\brief Load script
		///\param _rkScript Reference to script object
		size_t LoadScript(const CScript& _rkScript)
		{
			return AddScript(_rkScript);
		}
		///\brief Show variable state (debugging)
        void ShowVariableState() const    
		{ 
			ExposeVariableState(m_kState); 
		}
	};

}



